/*************************************************************************************
 * (c) 2003, Roberto A. Foglietta <robang@libero.it>
 *     Released under Licence GPL v2.0
 *     This program is provided "as is" without warranty of any kind.  
 *     Use it at your own risk.
 *************************************************************************************/

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>

#include "interface.h"
#include "support.h"

#define __MAIN_C
#include "main.h"



#include <stdio.h> 
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>

int
user_host_installed_keys_list(const char *str1, int flag)
{
	static GList *lista = NULL;
	static GList *vuota = NULL;
	static char *filename;
	static int changed = 0;
	static int visible = 1;

	switch(flag) {
	case APPEND:
		changed = 1;
		if(str1 != NULL && user_host_installed_keys_list_compare(str1)) {
			lista = g_list_append (lista, (gpointer) g_strdup(str1));	
			g_print("appended: %s\n", str1);
			return 0;
		}
		break;
	
	case UPDATE:
		if(!vuota) vuota = g_list_append (vuota, g_strdup(""));
		gtk_combo_set_popdown_strings (GTK_COMBO (lookup_widget(gtksshlogin, "combo_username")), (visible)?lista:vuota);
		break;
	
	case COMPARE:
		if(g_list_find(lista, (gpointer)str1))
			return 0;
		break;
		
	case SAVEDOWN:
		if(!changed)
			break;
		
		g_print("%d: %s\n", changed, filename);
		FILE *fp = fopen(filename, "wt");
		if(fp) {
			GList *tmp = lista;
			while(tmp) {
				fprintf(fp, "%s\n", (char *)(tmp->data));
				tmp = tmp->next;
			}
			fclose(fp);
			return 0;
		}
		break;
	
	case LOADINIT:
		user_host_installed_keys_list_resetall();
	
		char *home = getenv("HOME");
		if(str1 && home && home[0]) {
			struct stat buf;
			
			if(filename) 
				free(filename);
			filename = (char *)malloc(strlen(home) + strlen(str1) + 16);
			if(!filename)
				return 1;
			
			sprintf(filename,"%s/%s", home, str1);
			if(stat(filename, &buf)) 
				return 2;
			
			g_print("\n filename: %s\n", filename);
			FILE *fp = fopen(filename, "r");
			if(!fp) 
				return 3;
			
			int i = 0;	
			char *p, str[256];
			while( fscanf(fp,"%255s", str) != EOF && i < MAX_KEYS) {
				p = str; p += strlen(p); while(*p == ' ') { *p = 0; p--; }
				p = str; while(*p == ' ') p++;
				if(strlen(p) > 3) {
					g_print("\n%d:letto --> %s\n", i, p);
					user_host_installed_keys_list_append(p);
				}
				i++;
			}
			fclose(fp);
			
			changed = 0;
			return 0;
		}
		break;
		
	case RESETALL:
		changed = 1;
		if(lista) {
			g_list_foreach(lista, (GFunc)g_free, NULL);
			g_list_free(lista);
			lista = NULL;
		}
		lista = g_list_append (lista, (gpointer) strdup(""));
		break;

	case HIDESHOW:
		visible = !str1;
		user_host_installed_keys_list_update();
		break;
	}
	
	return 1;
}


int 
check_keys(void)
{
	struct stat buf;
	int dir;
	char *home, path[256];
	
	home = getenv("HOME");
	if(home) {
		sprintf(path,"%s/.ssh", home);
		dir = stat(path, &buf);
		g_print("\n stat '%s' in progress...\n", path);
		
		if(S_ISDIR(buf.st_mode) != 1) 
			return 1;
		
		sprintf(path,"%s/id_dsa", path);
		int priv = access(path, R_OK | F_OK);
		sprintf(path,"%s.pub", path);
		int publ = access(path, R_OK | F_OK);
		
		g_print("\n stat %s returned %d [%d] %d\n", home, dir, S_ISDIR(buf.st_mode), priv + publ);
		return priv + publ;
	} else
		return -1;
}

#include <string.h>
int
check_ssh_askpass(void) 
{
  char *sshask = getenv("SSH_ASKPASS");
  g_print("\n%s\n", sshask);
  if(sshask == NULL) 
	return 1;
  
  if(access(sshask, X_OK | F_OK))
	return -2;
  
  if(stringa_di_campo[CM_ASKPASS] == NULL)
	  stringa_di_campo[CM_ASKPASS] = strdup(sshask);

  g_print("\n%s is runable\n", sshask);
  return 0;
}
  

/********************************************************************/

int
main (int argc, char *argv[])
{
  gtk_set_locale ();
  gtk_init (&argc, &argv);

  add_pixmap_directory (PACKAGE_DATA_DIR "/pixmaps");
  add_pixmap_directory (PACKAGE_SOURCE_DIR "/pixmaps");

  /*
   */ 
  RESET_FLAG_DI_CAMPO();
  message = NULL;

  	
  /*
   * The following code was added by Glade to create one of each component
   * (except popup menus), just so that you see something after building
   * the project. Delete any components that you don't want shown initially.
   */
	
	
  gtksshlogin  = create_gtksshlogin ();
  gtk_widget_hide(lookup_widget(gtksshlogin, "label_password"));
  gtk_widget_hide(lookup_widget(gtksshlogin, "entry_password"));
  flag_di_campo[CM_ASKPASS] = !(check_ssh_askpass());
  g_print("\nflag_di_campo[CM_ASKPASS] = %d\n",  flag_di_campo[CM_ASKPASS]);
  if(check_keys()) {
	gtk_widget_show(lookup_widget(gtksshlogin, "label_genera"));
	gtk_widget_show(lookup_widget(gtksshlogin, "button_genera"));
	if(flag_di_campo[CM_ASKPASS]) {
		g_print("\nattivo l'uso delle passphrase\n");
		gtk_widget_show (lookup_widget(gtksshlogin, "checkbutton_passphrase"));
		gtk_widget_show (lookup_widget(gtksshlogin, "label_passphrase"));
		gtk_widget_set_sensitive (lookup_widget(gtksshlogin, "checkbutton_passphrase"), TRUE);
		gtk_widget_set_sensitive (lookup_widget(gtksshlogin, "label_passphrase"), TRUE);
	}
  } else {
	gtk_widget_show(lookup_widget(gtksshlogin, "label_installa"));
	gtk_widget_show(lookup_widget(gtksshlogin, "button_installa"));
  }

  /* RAF 2003-08-26: menu ancora da fare */
  char *nomi[] = {"altre_opzioni", "sessione_interattiva", "righe_di_history", "righe", "colonne", "nero_su_bianco", "xvt", "reinstalla_chiave", "rigenera_chiavi", "aggiungi_chiave", "aggiungi_amano", "" };
  int i = 0; 
  while(nomi[i][0])
	  gtk_widget_set_sensitive (lookup_widget(gtksshlogin, nomi[i++]), FALSE);
  
  
  user_host_installed_keys_list_loadinit(".gtksshloginrc");
  user_host_installed_keys_list_update();
  
  MOSTRA_AL_CENTRO(gtksshlogin);

  
  gtk_main ();
  return 0;
}

